/********************************************
Program                 : Temperature Sensor,
                          ADC, USART
Compiler                : WinAVR
Chip type               : ATmega32u4
System Clock            : 16 MHz
********************************************/


#include <avr/io.h>


unsigned char usart_data = 0b00000000;


void USART_Init( unsigned int baud )
{
    /* Set baud rate */
    UBRR1H = (unsigned char)(baud>>8);
    UBRR1L = (unsigned char)baud;

    /* Enable receiver and transmitter */
    UCSR1B = ((1<<RXEN1)|(1<<TXEN1));

    /* Set frame format: 8 data, 1 stop bit */
    UCSR1C = ((1<<UCSZ11)|(1<<UCSZ10));
}


void USART_Transmit( void )
{
    /* Wait for empty transmit buffer */
    while ( !( UCSR1A & (1<<UDRE1)) );
    /* Put data into buffer, sends the data */
    UDR1 = usart_data;
}


void USART_Receive( void )
{
    /* Wait for data to be received */
    while ( !(UCSR1A & (1<<RXC1)) );
    /* Get received data from buffer */
    usart_data = UDR1;
}


int main( void )
{
    // System Clock Prescaler initialization

    // Clock Prescale Register
    // Bits: CLKPCE - - - CLKPS3 CLKPS2 CLKPS1 CLKPS0
    CLKPR = 0b10000000;
    CLKPR = 0b00000000; // clk/1
    //CLKPR = 0b00000001; // clk/2
    //CLKPR = 0b00000010; // clk/4
    //CLKPR = 0b00000011; // clk/8
    //CLKPR = 0b00000100; // clk/16


    // Input/Output Ports initialization

    // Port B initialization
    // PB7, PB6, PB5, PB4, PB3, PB2, PB1, PB0
    DDRB = 0b00000000;
    PORTB = 0b00000000;

    // Port C initialization
    // PC7, PC6
    DDRC = 0b00000000;
    PORTC = 0b00000000;

    // Port D initialization
    // PD7, PD6, PD5, PD4, PD3, PD2, PD1, PD0
    DDRD = 0b00000000;
    PORTD = 0b00000000;

    // Port E initialization
    // PE6, PE2
    DDRE = 0b00000000;
    PORTE = 0b00000000;

    // Port F initialization
    // PF7, PF6, PF5, PF4, PF1, PF0
    DDRF = 0b00000000;
    PORTF = 0b00000000;


    // JTAG Interface Disable
    MCUCR |= 0b10000000;
    MCUCR |= 0b10000000;


    // USART initialization

    // RxD (PD2)
    // TxD (PD3)

    // Baud Rate 9600 bps
    USART_Init(103);


    // ADC initialization

    //===========================================================
    //
    // (GND) --- (10k-Resister) ----|---- (Thermistor) --- (VCC)
    //                              |
    //                          ADC0 (PF0)
    //
    //===========================================================

    // ADC Control and Status Register A
    // ADEN ADSC ADATE ADIF ADIE ADPSE2 ADPSE1 ADPSE0
    ADCSRA = 0b10000111;

    // ADC Control and Status Register B
    // ADHSM ACME MUX5 - ADTS3 ADTS2 ADTS1 ADTS0
    ADCSRB = 0b00000000;

    // ADC Multiplexer Selection Register
    // REFS1 REFS0 ADLAR MUX4 MUX3 MUX2 MUX1 MUX0
    ADMUX = 0b00000000;

    // Digital Input Disable Register 0
    // ADC7D ADC6D ADC5D ADC4D - - ADC1D ADC0D
    DIDR0 = 0b00000001;

    // Digital Input Disable Register 2
    // - - ADC13D ADC12D ADC11D ADC10D ADC9D ADC8D
    DIDR2 = 0b00000000;


    while (1)
    {
        USART_Receive();

        ADCSRA |= (1<<ADSC); // Start conversion
        while ( ADCSRA & (1<<ADSC) ); // Wait the conversition is complete

        usart_data = ADCL;
        USART_Transmit();

        usart_data = ADCH;
        USART_Transmit();
    }

    return 0;
}
